// Copyright (c) Microsoft. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for full license information.
var os = require('os');
var Q = require('q');
var readline = require("readline"), read = require('read'), async = require('async'), argparser = require('minimist'), trace = require('./trace');
var args = argparser(process.argv.slice(2));
function Qcheck(args, options, requiredArguments, optionalArguments) {
    var defer = Q.defer();
    this.check(args, options, requiredArguments, optionalArguments, function (err, result) {
        if (err) {
            defer.reject(err);
        }
        defer.resolve(result);
    });
    return defer.promise;
}
exports.Qcheck = Qcheck;
function check(args, options, requiredArguments, optionalArguments, done) {
    trace('inputs.check');
    var allArguments = {};
    for (var i = 0; i < requiredArguments.length; i++) {
        var arg = requiredArguments[i];
        var name = arg.name;
        allArguments[name] = args[i] ? arg.getValueFromString(args[i].toString()) : (options[name] ? arg.getValueFromString(options[name].toString()) : arg.defaultValue);
        if (!allArguments[name]) {
            trace('Required parameter ' + name + ' not supplied.');
            done(new Error('Required parameter ' + name + ' not supplied.' + os.EOL + 'Try adding a switch to the end of your command: --' + name + ' <' + arg.friendlyName + '>'), null);
        }
    }
    for (var i = 0; i < optionalArguments.length; i++) {
        var arg = optionalArguments[i];
        var name = arg.name;
        allArguments[name] = options[name] ? arg.getValueFromString(options[name].toString()) : arg.defaultValue;
    }
    done(null, allArguments);
}
exports.check = check;
// Q wrapper
function Qprompt(requiredInputs, optionalInputs) {
    var defer = Q.defer();
    this.prompt(requiredInputs, optionalInputs, function (err, result) {
        if (err) {
            defer.reject(err);
        }
        defer.resolve(result);
    });
    return defer.promise;
}
exports.Qprompt = Qprompt;
// done(err, result)
function prompt(requiredInputs, optionalInputs, done) {
    trace('inputs.prompt');
    var result = {};
    result['_'] = args['_'];
    var inputs = requiredInputs.concat(optionalInputs);
    // TODO: get rid of async and code just with Q
    async.forEachSeries(inputs, function (input, inputDone) {
        if (args[input.name]) {
            result[input.name] = args[input.name];
            inputDone(null, null);
            return;
        }
        var msg = 'Enter ' + input.friendlyName;
        if (input.defaultValue) {
            msg += ' (enter sets ' + input.defaultValue + ') ';
        }
        msg += ' > ';
        read({ prompt: msg, silent: input.silent }, function (err, answer) {
            result[input.name] = answer ? input.getValueFromString(answer.toString()) : input.defaultValue;
            trace('read: ' + result[input.name]);
            inputDone(null, null);
        });
    }, function (err) {
        if (err) {
            trace('Input reading failed with message: ' + err.message);
            done(err, null);
            return;
        }
        // final validation
        requiredInputs.forEach(function (input) {
            if (!result[input.name]) {
                trace(input.name + ' is required.');
                done(new Error(input.name + ' is required.'), null);
                return;
            }
        });
        done(null, result);
    });
}
exports.prompt = prompt;
